"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.executeMiddleware = void 0;
const deferred_promise_1 = require("@open-draft/deferred-promise");
const lodash_1 = require("lodash");
const cloneDeep_1 = __importDefault(require("lodash/cloneDeep"));
const actions_1 = require("./actions");
const targets_1 = require("./targets");
async function executeMiddleware(middleware, params, target) {
    const originalParams = (0, cloneDeep_1.default)(params);
    const paramsUpdatedPromise = new deferred_promise_1.DeferredPromise();
    const nextPromise = new deferred_promise_1.DeferredPromise();
    const result = middleware(params, (updatedParams) => {
        paramsUpdatedPromise.resolve(updatedParams);
        return nextPromise;
    }).catch((e) => {
        // reject params updated callback so it throws when awaited
        paramsUpdatedPromise.reject(e);
        // if next has already been resolved we must throw
        if (nextPromise.state === "fulfilled") {
            throw e;
        }
    });
    const updatedParams = await paramsUpdatedPromise;
    // execute middleware with updated params if action has changed
    if (updatedParams.action !== originalParams.action) {
        return executeMiddleware(middleware, updatedParams, (0, lodash_1.omit)(target, "index"));
    }
    if ((0, actions_1.isWriteAction)(updatedParams.action) && (0, targets_1.isWriteTarget)(target)) {
        return {
            nextPromise,
            result,
            updatedParams,
            origin: target,
            target: {
                action: updatedParams.action,
                field: target.field,
                relationName: target.relationName,
                index: target.index,
            },
        };
    }
    if ((0, actions_1.isReadAction)(updatedParams.action) && (0, targets_1.isReadTarget)(target)) {
        return {
            nextPromise,
            result,
            updatedParams,
            origin: target,
            target: {
                action: updatedParams.action,
                field: target.field,
                relationName: target.relationName,
            },
        };
    }
    throw new Error("Invalid target and params combination");
}
exports.executeMiddleware = executeMiddleware;
