"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setNestedResult = exports.getNestedResult = void 0;
const get_1 = __importDefault(require("lodash/get"));
const set_1 = __importDefault(require("lodash/set"));
const parentSymbol = Symbol("parent");
function addParentToResult(parent, result) {
    if (!Array.isArray(result)) {
        return { ...result, [parentSymbol]: parent };
    }
    return result.map((item) => ({ ...item, [parentSymbol]: parent }));
}
function removeParentFromResult(result) {
    if (!Array.isArray(result)) {
        const { [parentSymbol]: _, ...rest } = result;
        return rest;
    }
    return result.map(({ [parentSymbol]: _, ...rest }) => rest);
}
function getNestedResult(result, targetPath) {
    if (!Array.isArray(result)) {
        return (0, get_1.default)(result, targetPath);
    }
    return result.reduce((acc, item) => {
        const itemResult = (0, get_1.default)(item, targetPath);
        if (typeof itemResult !== "object" || itemResult === null) {
            return acc;
        }
        return acc.concat(addParentToResult(item, itemResult));
    }, []);
}
exports.getNestedResult = getNestedResult;
function setNestedResult(result, targetPath, modifiedResult) {
    if (!Array.isArray(result)) {
        return (0, set_1.default)(result, targetPath, modifiedResult);
    }
    result.forEach((item) => {
        const originalResult = (0, get_1.default)(item, targetPath);
        // if original result was an array we need to filter the result to match
        if (Array.isArray(originalResult)) {
            return (0, set_1.default)(item, targetPath, removeParentFromResult(modifiedResult.filter((modifiedItem) => modifiedItem[parentSymbol] === item)));
        }
        // if the orginal result was not an array we can just set the result
        const modifiedResultItem = modifiedResult.find(({ [parentSymbol]: parent }) => parent === item);
        return (0, set_1.default)(item, targetPath, modifiedResultItem ? removeParentFromResult(modifiedResultItem) : null);
    });
}
exports.setNestedResult = setNestedResult;
